"""
apeek main
"""

import pydub
import apeek
from apeek import WaveformData, ScalingFactor, default_settings

import sys
import shutil
import math
import optparse

def main():
    
    console_width = shutil.get_terminal_size((80,20))[0]

    parser = optparse.OptionParser(version=apeek.__version__)
    parser.add_option("-w","--width", help="Waveform width [defaults to terminal width]",
                      type=int,
                      metavar="WIDTH", default=console_width)
    parser.add_option("-l","--lines", help="Waveform height in lines",
                      dest="height", type=int,
                      metavar="HEIGHT", default=4)
    parser.add_option("-s","--scaling",help="Set scaling mode (linear, root) [defaults to root]",
                      dest="scaling", choices=("linear","root"),
                      default="root")
    parser.add_option("-i","--info", help="Print statistics", 
                      default=False, 
                      action='store_true')
    parser.add_option("-a","--absolute", help="Do not normalize waveform",
                      default=False, action='store_true')

    (options, args) = parser.parse_args()

    for file in args:
        audio = pydub.AudioSegment.from_file(file)
        settings = default_settings()

        if options.scaling == "linear":
            settings['scaling'] = ScalingFactor.LINEAR
        else:
            settings['scaling'] = ScalingFactor.ROOT
        
        settings['normalized'] = True

        if options.absolute:
            settings['normalized'] = False

        result = WaveformData.create_waveform_data(audio, time_bin_count=options.width, settings=settings)
        text = apeek.unicode_waveform(result.value_pairs, height=options.height)

        if len(args) > 1:
            print(file + ":")

        sys.stdout.write(text)
        sys.stdout.write("\n")

        if options.info:
            peak_db = 20 * math.log10(result.max_value / result.full_code_value)
            if result.max_index > 6:
                print(" " * (result.max_index - 4) + "max ↑" )
            else:
                print(" " * (result.max_index) + "↑ max" )

            print(f"DUR: {audio.frame_count() / audio.frame_rate:.2f} seconds @ {audio.frame_rate}Fs // " +
            f"Peak: {peak_db:.2f} dBFS at {result.max_sample / audio.frame_count():.2f} secs")


if __name__ == "__main__":
    main()



"""
apeek is a package for creating audio waveform previews.
"""

__version__ = "0.2.0"

from typing import List

import numpy as np

from .waveform import WaveformData, WaveformDataSettings, ScalingFactor, default_settings

def rectified_unicode_waveform_charmatrix(data: np.ndarray, height: int, bg: str = "·") -> List[List[str]]:
    """ 
    Create a grid of strs for a rectified waveform bargraph
    """
    BLOCK_CHARS = list(f"{bg}▁▂▃▄▅▆▇")
    FULL_BLOCK = "█"
    levels = height * (len(BLOCK_CHARS))

    retval = []

    for _, sample_pair in enumerate(data):
        rectified_value = max(abs(sample_pair[0]), abs(sample_pair[1]))
        leveled_value = int(rectified_value * levels)
        
        full, partial = divmod(leveled_value, len(BLOCK_CHARS))
        blank = height - full

        retval += [list( FULL_BLOCK * full + BLOCK_CHARS[partial] + BLOCK_CHARS[0] * blank)]

    return retval


def unicode_waveform(data: np.ndarray, height: int) -> str:
    
    char_matrix = rectified_unicode_waveform_charmatrix(data, height)
    
    # transpose
    retval_lines = list()
    for j in reversed(range(height)):
        this_line = ""
        for i in range(len(char_matrix)):
            this_line = this_line + char_matrix[i][j]

        retval_lines += [this_line]

    return "\n".join(retval_lines)





import numpy as np
from pydub import AudioSegment

from enum import Enum
from dataclasses import dataclass
from typing import TypedDict, Optional, cast


class ScalingFactor(Enum):
    LINEAR = 0
    ROOT = 1


class WaveformDataSettings(TypedDict):
    scaling: 'ScalingFactor'
    normalized: bool
    

def default_settings():
    return WaveformDataSettings(scaling=ScalingFactor.ROOT, normalized=True)


@dataclass
class WaveformData:
    """
    Data collected from an AudioSegment to be used for drawing a waveform.
    """
    value_pairs: np.ndarray 
    max_index: int
    max_sample: int
    max_value: float
    full_code_value: float
    samples_per_bin: int

    @classmethod
    def create_waveform_data(cls,
                             audio: AudioSegment, 
                             time_bin_count: Optional[int],
                             bin_length: Optional[int] = None,
                             settings: WaveformDataSettings = default_settings()) -> 'WaveformData':
        """
        Create a numpy array for use in drawing a waveform overview.
     
        """

        samples = audio.get_array_of_samples()
        channel_count = audio.channels
        assert isinstance(channel_count,int)
        
        frame_count = len(samples) // channel_count

        if time_bin_count is not None:
            window_length = (frame_count // time_bin_count) * channel_count
        elif bin_length is not None:
            window_length = bin_length
            time_bin_count = (frame_count // bin_length) * channel_count
        else:
            assert False

        retval = np.zeros((time_bin_count, 2))
        
        max_index = 0
        max_value = 0.0
        max_sample = 0
        for i in range(time_bin_count):
            window = samples[i * window_length : (i + 1) * window_length]
            retval[i] = (np.max(window),np.min(window))
            magnitude = np.max(np.abs(window))
            if magnitude > max_value:
                max_index = i
                max_sample = cast(int, np.argmax(np.abs(window)) + i * window_length)
                max_value = magnitude


        bit_depth = audio.sample_width
        assert isinstance(bit_depth,int) 
        full_code_value = (2 ** (bit_depth * 8 - 1)) 
        retval = retval / full_code_value

        if settings['normalized']:
            sample_max = np.max(retval)
            sample_min = np.min(retval)
            scale_max = max(abs(sample_max), abs(sample_min))
            if scale_max != 0:
                retval = retval / scale_max

        if settings['scaling'] == ScalingFactor.ROOT:
            signs = np.sign(retval)
            absval = np.sqrt(np.fabs(retval))
            retval = absval * signs

        return cls(retval, max_index, max_sample, 
                   max_value, full_code_value, samples_per_bin=window_length)


